library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

entity KeyScan is
port(	clk					: in 	std_logic;
		key_row				: out std_logic_vector(7 downto 0);
		key_col				: in 	std_logic_vector(2 downto 0);
		LED					: out	std_logic_vector(7 downto 0));
end KeyScan;

architecture design of KeyScan is
	signal	clk_1khz		: std_logic;
	signal	key_val			: integer range 0 to 24;
	signal	row_val			: std_logic_vector(7 downto 0) := "00000001";
	signal	col_val			: std_logic_vector(2 downto 0);
begin
	
	process(clk)
--		variable cnt_1khz	: integer range 0 to 1000;
		variable cnt_1khz	: integer := 0;
	
	begin 
		if rising_edge(clk) then
			if cnt_1khz >= 499 then
				clk_1khz <= not clk_1khz;
				cnt_1khz := 0;
			else
				cnt_1khz := cnt_1khz + 1;
			end if;
		end if;
	end process;
	
	process(clk_1khz, row_val, key_val)
	begin
		if rising_edge(clk_1khz) then
			row_val(7 downto 1) <= row_val(6 downto 0);
			row_val(0) <= row_val(7);
		end if;
		key_row <= row_val;
	end process;	
		
	process(clk_1khz, row_val, key_val)
	begin
		if falling_edge(clk_1khz) then
			if row_val = "10000000" then
				case key_col is
					when "100" => key_val <= 24;
					when "010" => key_val <= 16;
					when "001" => key_val <= 8;
					when others => key_val <= 0;
				end case;
			elsif row_val = "01000000" then
				case key_col is
					when "100" => key_val <= 23;
					when "010" => key_val <= 15;
					when "001" => key_val <= 7;
					when others => key_val <= 0;
				end case;
			elsif row_val = "00100000" then
				case key_col is
					when "100" => key_val <= 22;
					when "010" => key_val <= 14;
					when "001" => key_val <= 6;
					when others => key_val <= 0;
				end case;
			elsif row_val = "00010000" then
				case key_col is
					when "100" => key_val <= 21;
					when "010" => key_val <= 13;
					when "001" => key_val <= 5;
					when others => key_val <= 0;
				end case;
			elsif row_val = "00001000" then
				case key_col is
					when "100" => key_val <= 20;
					when "010" => key_val <= 12;
					when "001" => key_val <= 4;
					when others => key_val <= 0;
				end case;
			elsif row_val = "00000100" then
				case key_col is
					when "100" => key_val <= 19;
					when "010" => key_val <= 11;
					when "001" => key_val <= 3;
					when others => key_val <= 0;
				end case;
			elsif row_val = "00000010" then
				case key_col is
					when "100" => key_val <= 18;
					when "010" => key_val <= 10;
					when "001" => key_val <= 2;
					when others => key_val <= 0;
				end case;			
			elsif row_val = "00000001" then
				case key_col is
					when "100" => key_val <= 17;
					when "010" => key_val <= 9;
					when "001" => key_val <= 1;
					when others => key_val <= 0;
				end case;
			else
				key_val <= 0;
			end if;
		end if;
		LED <= conv_std_logic_vector(key_val, 8);		
	end process;
end design;