library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

entity DotMatrix is
port(	clk 			: in std_logic;
	dot_data 	: out std_logic_vector(6 downto 0);
	col_out		: out std_logic_vector(3 downto 0));
end DotMatrix;

architecture design of DotMatrix is
	type 		matrix2D is array (4 downto 0) of std_logic_vector(6 downto 0);
	signal 	data			: std_logic;
	signal	clk_1hz		: std_logic;
	signal	clk_1khz		: std_logic;
	signal	dot_data0	: matrix2D;
	signal	col_no	: std_logic_vector(3 downto 0) := "0000";	

--	constant	dot_data1	: matrix2D := ("1101010",
--													"1001101",
--													"1011101",
--													"1001101",
--													"1001010");

--	constant	dot_data2	: matrix2D := ("1111100",
--													"0101100",
--													"0101111",
--													"0101100",
--													"1111100");

-- data bit '0'=>'1', '1'=>'0'
	constant	dot_data1	: matrix2D := ("0010101",
													"0110010",
													"0100010",
													"0110010",
													"0110101");

	constant	dot_data2	: matrix2D := ("0000011",
													"1010011",
													"1010000",
													"1010011",
													"0000011");
														
begin
	process(clk)
		variable cnt_1hz	: integer range 0 to 1000000;
	begin 
		if rising_edge(clk) then
			if cnt_1hz >= 499999 then
				clk_1hz <= not clk_1hz;
				cnt_1hz := 0;
				data <= not data;
			else
				cnt_1hz := cnt_1hz + 1;
			end if;
		end if;
	end process;	

	process(clk)
		variable cnt_1khz	: integer range 0 to 13500;
	begin 
		if rising_edge(clk) then
			if cnt_1khz >= 499 then
				clk_1khz <= not clk_1khz;
				cnt_1khz := 0;
			else
				cnt_1khz := cnt_1khz + 1;
			end if;
		end if;
	end process;	
	
	process(clk_1khz, col_no, dot_data0)
	begin
		if rising_edge(clk_1khz) then
			if col_no = "0100" then
				col_no <= "0000";
			else 
				col_no <= col_no + '1';
			end if;
		end if;
	end process;

	col_out <= col_no;
	
	process(col_no)
	begin
		if data = '0' then
			case col_no is
				when "0000" =>	dot_data <= dot_data1(0);
				when "0001" => dot_data <= dot_data1(1);
				when "0010" => dot_data <= dot_data1(2);
				when "0011" => dot_data <= dot_data1(3);
				when "0100" => dot_data <= dot_data1(4);
				when others => dot_data <= "1111111";
			end case;
		else
			case col_no is
				when "0000" => dot_data <= dot_data2(0);
				when "0001" => dot_data <= dot_data2(1);
				when "0010" => dot_data <= dot_data2(2);
				when "0011" => dot_data <= dot_data2(3);
				when "0100" => dot_data <= dot_data2(4);
				when others => dot_data <= "1111111";
			end case;
		end if;	
	end process;
end design;