library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

entity TextLCD is
Port ( clk 		: in  std_logic;
	   SEL		: out std_logic;
		RS			: out std_logic;
		RW			: out std_logic;
		DB			: out std_logic_vector(7 downto 0);
		backlight	: out std_logic;
		reset		: in  std_logic);           
end TextLCD;

architecture design of TextLCD is
	signal clk_1khz 	: std_logic;
	signal lcd_cnt		: integer range 0 to 86 := 0;
	signal lcd_db		: std_logic_vector(7 downto 0) := "00000000";

begin	
	process(clk)
		variable clk_cnt	: integer range 0 to 13500;
	begin
		if rising_edge(clk) then
			if clk_cnt >= 499 then
				clk_cnt := 0;
				clk_1khz <= not clk_1khz;
			else
				clk_cnt := clk_cnt + 1;
			end if;
		end if;
	end process;

	backlight <= '1';	
	
	process(clk_1khz,reset)
	begin
		if(reset = '0') then
			lcd_cnt <= 0;
		elsif rising_edge(clk_1khz) then
			if(lcd_cnt >= 86 ) then				
				lcd_cnt <= 54;									
			else
				lcd_cnt <= lcd_cnt+1;
			end if;
		end if;
	end process;

	process(lcd_cnt)
	begin
		case lcd_cnt is
			-- rw = '0', rs = '0'
			when 50	=> lcd_db <= "00111000"; --Function set 
			when 51	=> lcd_db <= "00001100"; --Display on
			when 52 	=> lcd_db <= "00000001"; --clear display 
			when 53	=> lcd_db <= "00000110"; --entry mode set
			when 54	=> lcd_db <= "10000000"; -- DDRAM data send Line 1	

			-- rw = '0', rs = '1'			
			when 55	=> lcd_db <= "00100000"; --  blank
			when 56	=> lcd_db <= "01010111"; --   W
			when 57	=> lcd_db <= "01100101"; --   e	
			when 58	=> lcd_db <= "01101100"; --   l	
			when 59	=> lcd_db <= "01000011"; --   C	
			when 60	=> lcd_db <= "01101111"; --   o	
			when 61	=> lcd_db <= "01101101"; --   m	
			when 62	=> lcd_db <= "01100101"; --   e	
			when 63	=> lcd_db <= "00100000"; --   blank	
			when 64	=> lcd_db <= "01110100"; --   t	
			when 65	=> lcd_db <= "01101111"; --   o	
			when 66	=> lcd_db <= "00100000"; --   blank	
			when 67	=> lcd_db <= "00100000"; --   blank	

			-- rw = '0', rs = '0'
			when 68		=> lcd_db <= "11000000";  -- DDRAM data send Line 2			

			-- rw = '0', rs = '1'
			when 69	=> lcd_db <= "00100000"; --   blank	
			when 70	=> lcd_db <= "01000110"; --   F	
			when 71	=> lcd_db <= "01010000"; --   P	
			when 72	=> lcd_db <= "01000111"; --   G
			when 73	=> lcd_db <= "01000001"; --   A	
			when 74	=> lcd_db <= "00100000"; --   blank	
			when 75	=> lcd_db <= "00100000"; --   blank	
			when 76	=> lcd_db <= "01010111"; --   W
			when 77	=> lcd_db <= "01101111"; --   o	
			when 78	=> lcd_db <= "01110010"; --   r	
			when 79	=> lcd_db <= "01101100"; --   l	
			when 80	=> lcd_db <= "01100100"; --   d	
			when 81	=> lcd_db <= "00100001"; --   !	
			when 82	=> lcd_db <= "00100001"; --   !	
			when 83	=> lcd_db <= "00100001"; --   !	
			when 84	=> lcd_db <= "00100001"; --   !	
			when others => lcd_db <= "00000000";								
		end case;
	end process;	
	RW <= '0';															
	RS <= '0' when (lcd_cnt >= 0 and lcd_cnt <= 54) or (lcd_cnt = 68) else '1';
	SEL <= clk_1khz when (lcd_cnt >= 50) and (lcd_cnt <= 84) else '0';
	DB <= lcd_db;	
end design;