-- Stepper Motor : 28BYJ-48) 
-- mode 00 : bipolar single phase mode
-- mode 01 : bipolar two phase mode
-- mode 10 : unipolar full step mode
-- mode 11 : unipolar half step mode

-- speed control 
-- speed 00 : stepClk=400Hz
-- speed 01 : stepClk=200Hz
-- speed 10 : stepClk=100Hz
-- speed 11 : stepClk=50Hz

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;

entity StepMotor is
port( clock, dir	: in 	std_logic;
		mode			: in std_logic_vector(1 downto 0);
		speed			: in 	unsigned(1 downto 0);
		stepOut		: out	std_logic_vector(3 downto 0);
		stepEn		: out std_logic_vector(1 downto 0);
		LED			: out std_logic_vector(2 downto 0);
		LED2			: out std_logic_vector(3 downto 0));
end StepMotor;

architecture design of StepMotor is
	signal step				: integer range 0 to 7;
	signal stepPins		: std_logic_vector(3 downto 0);
	signal stepClk			: std_logic;
	constant StepDivider : integer := 2500;	

begin 
	process(clock, step)
	variable cnt	: integer range 0 to 20000;
	begin
		if rising_edge(clock) then
			if (cnt >= StepDivider*(conv_integer(speed)+1)/2) then
				stepClk <= not stepClk;
				cnt := 0;
			else
				cnt := cnt + 1;
			end if;
		end if;
	end process;

	process(stepClk)
	begin
		if rising_edge(stepClk) then
			step <= step + 1;
		end if;	
	end process;
	
	process(stepClk, step, mode, dir)
	begin
-- bipolar single phase mode
			if (mode = "00") then
				if(dir = '0') then
					case step is
						when 0 => stepPins <= "1110";
						when 1 => stepPins <= "1101";
						when 2 => stepPins <= "1011";
						when 3 => stepPins <= "0111";
						when 4 => stepPins <= "1110";
						when 5 => stepPins <= "1101";
						when 6 => stepPins <= "1011";
						when 7 => stepPins <= "0111";
					end case;
				else
					case step is
						when 0 => stepPins <= "1110";
						when 1 => stepPins <= "0111";
						when 2 => stepPins <= "1011";
						when 3 => stepPins <= "1101";
						when 4 => stepPins <= "1110";
						when 5 => stepPins <= "0111";
						when 6 => stepPins <= "1011";
						when 7 => stepPins <= "1101";
					end case;
				end if;		-- end dir	
			end if;			-- end mode
			
-- bipolar two phase mode
			if (mode = "01") then
				if(dir = '0') then
					case step is
						when 0 => stepPins <= "1100";
						when 1 => stepPins <= "0110";
						when 2 => stepPins <= "0011";
						when 3 => stepPins <= "1001";
						when 4 => stepPins <= "1100";
						when 5 => stepPins <= "0110";
						when 6 => stepPins <= "0011";
						when 7 => stepPins <= "1001";
					end case;
				else
					case step is
						when 0 => stepPins <= "1100";
						when 1 => stepPins <= "1001";
						when 2 => stepPins <= "0011";
						when 3 => stepPins <= "0110";
						when 4 => stepPins <= "1100";
						when 5 => stepPins <= "1001";
						when 6 => stepPins <= "0011";
						when 7 => stepPins <= "0110";
					end case;
				end if;		-- end dir	
			end if;			-- end mode
			
-- unipolar full step mode
			if (mode = "10") then
				if(dir = '0') then
					case step is
						when 0 => stepPins <= "1110";
						when 1 => stepPins <= "1101";
						when 2 => stepPins <= "1011";
						when 3 => stepPins <= "0111";
						when 4 => stepPins <= "1110";
						when 5 => stepPins <= "1101";
						when 6 => stepPins <= "1011";
						when 7 => stepPins <= "0111";
					end case;
				else
					case step is
						when 0 => stepPins <= "1110";
						when 1 => stepPins <= "0111";
						when 2 => stepPins <= "1011";
						when 3 => stepPins <= "1101";
						when 4 => stepPins <= "1110";
						when 5 => stepPins <= "0111";
						when 6 => stepPins <= "1011";
						when 7 => stepPins <= "1101";
					end case;
				end if;		-- end dir	
			end if;			-- end mode
	
-- unipolar half step mode
			if (mode = "11") then
				if(dir = '0') then
					case step is
						when 0 => stepPins <= "1110";
						when 1 => stepPins <= "0110";
						when 2 => stepPins <= "0111";
						when 3 => stepPins <= "0011";
						when 4 => stepPins <= "1011";
						when 5 => stepPins <= "1001";
						when 6 => stepPins <= "1101";
						when 7 => stepPins <= "1100";
					end case;
				else
					case step is
						when 0 => stepPins <= "1110";
						when 1 => stepPins <= "1100";
						when 2 => stepPins <= "1101";
						when 3 => stepPins <= "1001";
						when 4 => stepPins <= "1011";
						when 5 => stepPins <= "0011";
						when 6 => stepPins <= "0111";
						when 7 => stepPins <= "0110";
					end case;
				end if;		-- end dir	
			end if;			-- end mode
	end process;
	stepEn <= "11";
	stepOut <= stepPins;
	LED <= conv_std_logic_vector(step, 3);
	LED2 <= stepPins;
end design;