//*** Stepper Motor : 28BYJ-48) ***//
// mode 00 : bipolar single phase mode
// mode 01 : bipolar two phase mode
// mode 10 : unipolar full step mode
// mode 11 : unipolar half step mode

//*** speed control ***//
// speed 00 : stepClk=400Hz
// speed 01 : stepClk=200Hz
// speed 10 : stepClk=100Hz
// speed 11 : stepClk=50Hz

module StepMotor(clock, mode, dir, speed, stepOut, stepEn, LED, LED2);
parameter StepDivider=2500; // every 400Hz
 
input clock;
input [1:0] mode;
input	dir;
input [1:0] speed;
output [3:0] stepOut;
output [1:0] stepEn;
output [2:0] LED;
output [3:0] LED2;

integer cnt=0;
reg	stepClk;
reg [3:0] stepPins;
reg [2:0] step; // 8 positions for half steps
 
always @ (posedge clock)
begin
	if (cnt >= StepDivider*(speed+1)/2) begin
			cnt <= 0;
			stepClk <= ~stepClk;
	end else
			cnt <= cnt+1;
end			
 
 
always @ (posedge stepClk)
begin
	step <= step + 1'b1;
end

always @ (step, mode, dir)
begin
// *** bipolar single phase mode
	if (mode == 2'b00)  
		if (dir == 1'b0) begin
			case(step)
										// OYPB  O(range)Y(ellow)P(ink)B(lue)    
				0: stepPins <= 4'b1110;
				1: stepPins <= 4'b1101;
				2: stepPins <= 4'b1011;
				3: stepPins <= 4'b0111;
				4: stepPins <= 4'b1110;
				5: stepPins <= 4'b1101;
				6: stepPins <= 4'b1011;
				7: stepPins <= 4'b0111;
			endcase
		end else begin
			case(step)
				0: stepPins <= 4'b1110;
				1: stepPins <= 4'b0111;
				2: stepPins <= 4'b1011;
				3: stepPins <= 4'b1101;
				4: stepPins <= 4'b1110;
				5: stepPins <= 4'b0111;
				6: stepPins <= 4'b1011;
				7: stepPins <= 4'b1101;
			endcase
		end	
			
// *** bipolar two phase mode	
	else if (mode == 2'b01) 
		if (dir == 1'b0) begin
			case(step)
				0: stepPins <= 4'b1100;
				1: stepPins <= 4'b0110;
				2: stepPins <= 4'b0011;
				3: stepPins <= 4'b1001;
				4: stepPins <= 4'b1100;
				5: stepPins <= 4'b0110;
				6: stepPins <= 4'b0011;
				7: stepPins <= 4'b1001;
			endcase
		end else begin
			case(step)
				0: stepPins <= 4'b1100;
				1: stepPins <= 4'b1001;
				2: stepPins <= 4'b0011;
				3: stepPins <= 4'b0110;
				4: stepPins <= 4'b1100;
				5: stepPins <= 4'b1001;
				6: stepPins <= 4'b0011;
				7: stepPins <= 4'b0110;
			endcase
		end	

// *** unipolar full step mode 
		else if (mode == 2'b10)
		if (dir == 1'b0) begin
			case(step)
				0: stepPins <= 4'b1110;
				1: stepPins <= 4'b1101;
				2: stepPins <= 4'b1011;
				3: stepPins <= 4'b0111;
				4: stepPins <= 4'b1110;
				5: stepPins <= 4'b1101;
				6: stepPins <= 4'b1011;
				7: stepPins <= 4'b0111;
			endcase
		end else begin
			case(step)
				0: stepPins <= 4'b1110;
				1: stepPins <= 4'b0111;
				2: stepPins <= 4'b1011;
				3: stepPins <= 4'b1101;
				4: stepPins <= 4'b1110;
				5: stepPins <= 4'b0111;
				6: stepPins <= 4'b1011;
				7: stepPins <= 4'b1101;
			endcase
		end	
			
// *** unipolar half step mode ==> 100Hz?서 1?전????40.6?	
	else
		if (dir == 1'b0) begin
			case(step)
				0: stepPins <= 4'b1110;
				1: stepPins <= 4'b0110;
				2: stepPins <= 4'b0111;
				3: stepPins <= 4'b0011;
				4: stepPins <= 4'b1011;
				5: stepPins <= 4'b1001;
				6: stepPins <= 4'b1101;
				7: stepPins <= 4'b1100;
			endcase
		end else begin
			case(step)
				0: stepPins <= 4'b1110;
				1: stepPins <= 4'b1100;
				2: stepPins <= 4'b1101;
				3: stepPins <= 4'b1001;
				4: stepPins <= 4'b1011;
				5: stepPins <= 4'b0011;
				6: stepPins <= 4'b0111;
				7: stepPins <= 4'b0110;
			endcase
		end	
end

assign stepEn = 2'b11;
assign stepOut = stepPins;
assign LED = step;
assign LED2 = stepPins;
endmodule