library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
use ieee.numeric_std.all;

entity Step_Motor is
port(	clock				: in 	std_logic;
		speed				: in  std_logic_vector(1 downto 0);
		dirIn				: in	std_logic;
		ms					: in 	std_logic;
		step_en			: out	std_logic;
		stepOut			: out	std_logic;
		dir				: out	std_logic;
		ms_out			: out	std_logic_vector(2 downto 0));
end Step_Motor;

architecture design of Step_Motor is
	signal		stepClk			: std_logic;
	signal		step				: std_logic;
	constant		StepDivider		: integer := 1250;	

begin
	process(clock)
	variable cnt	: integer range 0 to 10000;
	begin
		if rising_edge(clock) then
			if (cnt >= StepDivider*(conv_integer(speed)+1)/2) then
				stepClk <= not stepClk;
				cnt := 0;
			else
				cnt := cnt + 1;
			end if;
		end if;
	end process;
	
	process(stepClk, step)
	begin
		if rising_edge(stepClk) then
			step_en <= '0';
			step <= not step;
			if (dirIn = '1') then
				dir <= '1';
			else
				dir <= '0';
			end if;
		end if;
		stepOut <= step;
	end process;
	
	stepOut <= step;	
	ms_out(0) <= ms;
	ms_out(1) <= '0';
	ms_out(2) <= '0';
end design;	