library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

entity ServoMotor is
port(	clk				: in 	std_logic;
		sw_in			: in 	std_logic_vector(2 downto 0);
		ServoOut1 		: out	std_logic;
		ServoOut2       : out    std_logic);
end ServoMotor;

architecture design of ServoMotor is
	signal	clk_500KHz		: std_logic;
	signal	cpulse			: std_logic := '1'; -- center pulse 1.5 msec
	signal	rpulse			: std_logic := '1'; -- righ pulse 2.1 msec
	signal	lpulse			: std_logic := '1'; -- left pulse 0.9 msec
	signal	mode			: std_logic_vector(2 downto 0);
begin

-- switch input : sw(0) => reset, sw(1) => right turn, sw(2) => left turn

--	500KHz clock generation
	process(clk)
	begin 
		if rising_edge(clk) then
				clk_500KHz <= not clk_500KHz;
		end if;
	end process;
	
-- right direction pulse generation(2.1msec duration)	
	process(clk_500KHz)
		variable cnt_20ms 	: integer range 0 to 10000 := 0;
		variable cnt_1_5ms	: integer range 0 to 750 := 0;
		variable cnt_0_9ms	: integer range 0 to 450 := 0;
		variable cnt_2_1ms	: integer range 0 to 1050 := 0;	
	begin
		if rising_edge(clk_500KHz) then
			if cnt_20ms < 10000 then
				cnt_20ms := cnt_20ms + 1;
				if cnt_1_5ms < 750 then
					cnt_1_5ms := cnt_1_5ms + 1;
				else
					cpulse <= '0';
				end if;
					
				if cnt_0_9ms < 450 then
					cnt_0_9ms := cnt_0_9ms + 1;
				else
					lpulse <= '0';
				end if;
				
				if cnt_2_1ms < 1050 then
					cnt_2_1ms := cnt_2_1ms + 1;
				else
					rpulse <= '0';
				end if;
			else
				cnt_20ms := 0;
				cnt_1_5ms := 0;
				cnt_0_9ms := 0;
				cnt_2_1ms := 0;

				cpulse <= '1';
				lpulse <= '1';
				rpulse <= '1';
			end if;
		end if;
	end process;
	
	process(mode, sw_in)
	begin
		if rising_edge(clk) then
			mode <= sw_in;
			case mode is 
				when "100" => 
				    ServoOut1 <= cpulse;
				    ServoOut2 <= cpulse;
				when "010" => 
				    ServoOut1 <= lpulse;
				    ServoOut2 <= lpulse;
				when "001" => 
				    ServoOut1 <= rpulse;
				    ServoOut2 <= rpulse;
				when others => NULL;
			end case;
		end if;		
	end process;			
end design;		